#   MIDAS  Httpd Service for the Spectrum Server

global env

package require MIDASDomainHandler   ;#  load default support procedures

global MIDAS_SpectrumAccessServer_Exists
set MIDAS_SpectrumAccessServer_Exists 0

global MIDAS_SpectrumAccessServer_Verbose
set    MIDAS_SpectrumAccessServer_Verbose 0

#global MIDAS_SOAPDomainHandler_Verbose
#set MIDAS_SOAPDomainHandler_Verbose 0

#   This service implements a SOAP memsas
package require MemSasAccess

# Use namespaces to isolate your methods
namespace eval urn:SpectrumService {
  catch {namespace import ::Spectrum::Debug}
  catch {namespace import ::Spectrum::Logging}
  catch {namespace import ::Spectrum::New1D}
  catch {namespace import ::Spectrum::New2D}
  catch {namespace import ::Spectrum::Delete}
  catch {namespace import ::Spectrum::Zero}
  catch {namespace import ::Spectrum::Read1DR}
  catch {namespace import ::Spectrum::Read1D}
  catch {namespace import ::Spectrum::Read1DText}
  catch {namespace import ::Spectrum::Read2D}
  catch {namespace import ::Spectrum::Read2DText}
  catch {namespace import ::Spectrum::Write1D}
  catch {namespace import ::Spectrum::Write2D}
  catch {namespace import ::Spectrum::Compressions}
  catch {namespace import ::Spectrum::Endianness}
  catch {namespace import ::Spectrum::Error}

  catch {namespace import ::base64::encode}
  catch {namespace import ::base64::decode}

  set SpectrumIndex 1

  proc ServerVerbose {Option} {
        global MIDAS_SpectrumAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose
        set MIDAS_SpectrumAccessServer_Verbose $Option
        set MIDAS_SOAPDomainHandler_Verbose $Option
        set z [catch {Logging $Option} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecVerbose detected error in call to Spectrum::Logging"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
  }

    proc SpecDebug {Flag} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecDebug with parameter $Flag"}
        set z [catch {Debug $Flag} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDebug detected error in call to Spectrum::Debug"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecLogging {Value} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecLogging with parameter $Value"}
        set z [catch {Logging $Value} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecLogging detected error in call to Spectrum::Logging"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNew {Spectrum b r args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecNew with parameters $Spectrum $b $r $args, (len: [llength $args])"}
        set t [third $args]
        if {[llength $args]==0 || [second $args]==0 || [second $args]==""} {
          set z [catch {LocalSpecNew1D $Spectrum $b $r $t} m]
          if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew detected error in call to Spectrum::New1D rc=$z"}
        } else {
          set z [catch {LocalSpecNew2D $Spectrum $b $r [first $args] [second $args] $t} m]
          if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew detected error in call to Spectrum::New2D rc=$z"}
        }

        return $m
    }

    proc SpecNew1D {Spectrum b r args} {
        return [LocalSpecNew1D $Spectrum $b $r [first [first $args]]]
    }

    proc SpecNew2D {Spectrum b0 r0 b1 r1 args} {
        return [LocalSpecNew2D $Spectrum $b0 $r0 $b1 $r1 [first [first $args]]]
    }

    proc LocalSpecNew1D {Spectrum b r t} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar 2 SpectrumIndex SpectrumIndex
        upvar 2 Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecNew1D with parameters $Spectrum $b $r $t"}

#    Check Spectrum name is not already in use
        if {[info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Spectrum name already in use"
        }

        if {$t == ""} {set t 5} { if {$t < 0 || $t > 6} {set t 5} }

        set Spectra($Spectrum) $SpectrumIndex

        upvar 2 S$SpectrumIndex S
        set S(dim) 1
        set S(base_0) $b
        set S(range_0) $r
        set S(type) $t

        incr SpectrumIndex

        set z [catch {New1D $Spectrum $b $r $t} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew1D detected error in call to Spectrum::New1D rc=$rc"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc LocalSpecNew2D {Spectrum b0 r0 b1 r1 t} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar 2 SpectrumIndex SpectrumIndex
        upvar 2 Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecNew2D with parameters $Spectrum $b0 $r0 $b1 $r1 $t"}

#    Check Spectrum name is not already in use
        if {[info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Spectrum name already in use"
        }

        set Spectra($Spectrum) $SpectrumIndex

        if {$t == ""} {set t 5} { if {$t < 0 || $t > 6} {set t 5} }

        upvar 2 S$SpectrumIndex S
        set S(dim) 2
        set S(base_0) $b0
        set S(range_0) $r0
        set S(base_1) $b1
        set S(range_1) $r1
        set S(type) $t

        incr SpectrumIndex

        set z [catch {New2D $Spectrum $b0 $r0 $b1 $r1 $t} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecNew2D detected error in call to Spectrum::New2D rc=$rc"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecDelete {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecDelete with parameter $Spectrum"}

        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode 0x30004 -errorinfo "Spectrum name does not exist"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        array unset S

        unset Spectra($Spectrum)

        set z [catch {Delete $Spectrum} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecDelete detected error in call to Spectrum::Delete"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecZero {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecZero with parameter $Spectrum"}
        set z [catch {Zero $Spectrum} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecZero detected error in call to Spectrum::Zero"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecDetails {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecDetails with parameters $Spectrum"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set m {}
        set d $S(dim)
        lappend m $d

        set b {}
        for {set i 0} {$i < $d} {incr i} {
           lappend b $S(base_$i)
        }
        lappend m $b

        set r {}
        for {set i 0} {$i < $d} {incr i} {
           lappend r $S(range_$i)
        }
        lappend m $r

        lappend m {0 0} [list $S(type) $S(type)]

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Endianness with parameters $Spectrum"}

        set z [catch {Endianness $Spectrum} e]
        if {$z != 0} then {return -code error -errorcode "$e" -errorinfo "SpecDetails detected error in call to Spectrum::Endianness"}
        lappend m $e

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecNames {pattern} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecNames with parameter $pattern"}

        set m {}
        if {[array exists Spectra]} then {
		if {[llength $pattern] == 1} {set pattern [first $pattern]}
		if {[llength $pattern] == 0} {set pattern "*"}
	        foreach Spectrum [array names Spectra -glob $pattern] {
	           lappend m [list "1" "$Spectrum"]
	        }
#	} else {
#             return -code error -errorcode 0x30004 -errorinfo "Spectrum names do not exist"
	}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecRead1DText {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecRead1DText with parameters $Spectrum $args"}
        if {[llength $args] < 2} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead1DText: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
           if {[llength $args] > 2} {set s [lindex $args 2]} else {set s 1}
           if {[llength $args] > 3} {set t [lindex $args 3]} else {
              upvar Spectra Spectra
              set Index $Spectra($Spectrum)
              upvar S$Index S
              set t $S(type)
           }
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read1DText with parameters $Spectrum $b $r $s $t"}
        set z [catch {Read1DText $Spectrum $b $r $s $t} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead1DText detected error in call to Spectrum::Read1DText"}

#        set status "$d"
        return [rpcvar::rpcvar string $d]
    }

    proc SpecRead1D {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecRead1D with parameters $Spectrum $args"}

        if {[llength $args] < 2} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead1D: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
           if {[llength $args] > 2} {set s [lindex $args 2]} else {set s 1}
           if {[llength $args] > 3} {set t [lindex $args 3]
if {$MIDAS_SpectrumAccessServer_Verbose} {puts "read as requested type $t"}
} else {
              upvar Spectra Spectra
              set Index $Spectra($Spectrum)
              upvar S$Index S
              set t $S(type)
if {$MIDAS_SpectrumAccessServer_Verbose} {puts "read as intrinsic type $t"}
           }
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read1D with parameters $Spectrum $b $r $s $t"}
        set z [catch {Read1D $Spectrum $b $r $s $t} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead1D detected error in call to Spectrum::Read1D"}

#        set status "$d"
        return [rpcvar::rpcvar base64Binary $d]
    }

    proc SpecRead1DR {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecRead1DR with parameters $Spectrum $args"}
        if {[llength $args] < 2} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead1DR: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
           if {[llength $args] > 2} {set s [lindex $args 2]} else {set s 1}
           if {[llength $args] > 3} {set t [lindex $args 3]} else {
              upvar Spectra Spectra
              set Index $Spectra($Spectrum)
              upvar S$Index S
              set t $S(type)
           }
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read1DR with parameters $Spectrum $b $r $s $t"}
        set z [catch {Read1DR $Spectrum $b $r $s $t} m]
        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecRead1DR detected error in call to Spectrum::Read1DR"}

#        set status "$m"
        return [rpcvar::rpcvar base64Binary $m]
    }

    proc SpecRead2D {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecRead2D with parameters $Spectrum $args"}
        if {[llength $args] < 4} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead2D: not enough parameters"
        } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
           if {[llength $args] > 4} {set s1 [lindex $args 4]} else {set s1 1}
           if {[llength $args] > 5} {set s2 [lindex $args 5]} else {set s2 1}
           if {[llength $args] > 6} {set t [lindex $args 6]} else {
              upvar Spectra Spectra
              set Index $Spectra($Spectrum)
              upvar S$Index S
              set t $S(type)
           }
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read2D with parameters $Spectrum $b1 $r1 $b2 $r2 $s1 $s2 $t"}
        set z [catch {Read2D $Spectrum $b1 $r1 $b2 $r2 $s1 $s2 $t} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead2D detected error in call to Spectrum::Read2D"}

        set status "$d"
        return [rpcvar::rpcvar base64Binary $status]
    }

    proc SpecRead2DText {Spectrum args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecRead2DText with parameters $Spectrum $args"}
        if {[llength $args] < 4} {
           return -code error -errorcode "0x30005" -errorinfo "SpecRead2DText: not enough parameters"
        } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
           if {[llength $args] > 4} {set s1 [lindex $args 4]} else {set s1 1}
           if {[llength $args] > 5} {set s2 [lindex $args 5]} else {set s2 1}
           if {[llength $args] > 6} {set t [lindex $args 6]} else {
              upvar Spectra Spectra
              set Index $Spectra($Spectrum)
              upvar S$Index S
              set t $S(type)
           }
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Read2DText with parameters $Spectrum $b1 $r1 $b2 $r2 $s1 $s2 $t"}
        set z [catch {Read2DText $Spectrum $b1 $r1 $b2 $r2 $s1 $s2 $t} d]
        if {$z != 0} then {return -code error -errorcode "$d" -errorinfo "SpecRead2DText detected error in call to Spectrum::Read2DText"}

        set status "$d"
        return [rpcvar::rpcvar string $status]
    }

     proc SpecWrite1D {Spectrum c args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecWrite1D with parameters $Spectrum (counts) $args"}
        if {$MIDAS_SpectrumAccessServer_Verbose && [string length $c] < 256} {puts "counts $c"}
        if {[llength $args] < 4} {
           return -code error -errorcode "0x30005" -errorinfo "SpecWrite1D: not enough parameters"
        } else {
           set b [lindex $args 0]
           set r [lindex $args 1]
           set t [lindex $args 2]
           set e [lindex $args 3]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Write1D with parameters $Spectrum $b $r (counts) $t $e"}
        set z [catch {decode $c} d]
        if {!$z} {
           set z [catch {Write1D $Spectrum $b $r $d $t $e} m]
           if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecWrite1D detected error in call to Spectrum::Write1D"}
        } else {set m $d}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecWrite2D {Spectrum c args} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecWrite2D with parameters $Spectrum (counts) $args"}
        if {$MIDAS_SpectrumAccessServer_Verbose && [string length $c] < 256} {puts "counts $c"}
        if {[llength $args] < 6} {
           return -code error -errorcode "0x30005" -errorinfo "SpecWrite2D: not enough parameters"
        } else {
           set b1 [lindex $args 0]
           set r1 [lindex $args 1]
           set b2 [lindex $args 2]
           set r2 [lindex $args 3]
           set t  [lindex $args 4]
           set e  [lindex $args 5]
        }
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "calling method Spectrum::Write2D with parameters $Spectrum $b1 $r1 $b2 $r2 (counts) $t $e"}
        set z [catch {decode $c} d]
        if {!$z} {
           set z [catch {Write2D $Spectrum $b1 $r1 $b2 $r2 $d $t $e} m]
           if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecWrite2D detected error in call to Spectrum::Write2D"}
        } else {set m $d}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecGetTitle {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecGetTitle with parameters $Spectrum"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {[info exists S(info_1)]} then {
           set m $S(info_1)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutTitle {Spectrum t} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecPutTitle with parameters $Spectrum $t"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set S(info_1) $t

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetLabel {Spectrum} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecGetLabel with parameters $Spectrum"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {[info exists S(info_2)]} then {
           set m $S(info_2)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutLabel {Spectrum l} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecPutLabel with parameters $Spectrum $l"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set S(info_2) $t

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetInfo {Spectrum n} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecGetInfo with parameters $Spectrum $n"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {[info exists S(info_$n)]} then {
           set m $S(info_$n)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]

    }

    proc SpecPutInfo {Spectrum n i} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecPutInfo with parameters $Spectrum $n $i"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        set S(info_$n) $i

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetAnnot {Spectrum d} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecGetAnnot with parameters $Spectrum $d"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Annotation string number out of range"
        }
        if {[info exists S(annot_$d)]} then {
           set m $S(annot_$d)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutAnnot {Spectrum d a} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecPutCalib with parameters $Spectrum $d $c"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Annotation string number out of range"
        }
        set S(annot_$d) $a

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetCalib {Spectrum d} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecGetCalib with parameters $Spectrum $d"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Calibration string number out of range"
        }
        if {[info exists S(calib_$d)]} then {
           set m $S(calib_$d)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutCalib {Spectrum d c} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecPutCalib with parameters $Spectrum $d $c"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Calibration string number out of range"
        }
        set S(calib_$d) $c

        return [rpcvar::rpcvar string ""]
    }

    proc SpecGetEffic {Spectrum d} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecGetEffic with parameters $Spectrum $d"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Efficiency string number out of range"
        }
        if {[info exists S(effic_$d)]} then {
           set m $S(effic_$d)
        } else {
           set m ""
        }

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecPutEffic {Spectrum d e} {
        global   MIDAS_SpectrumAccessServer_Verbose
        upvar Spectra Spectra

        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecPutEffic with parameters $Spectrum $d $e"}

#    Check Spectrum name is in use
        if {![info exists Spectra($Spectrum)]} then {
             return -code error -errorcode "0x30004" -errorinfo "Invalid spectrum name"
        }

        set Index $Spectra($Spectrum)
        upvar S$Index S
        if {$d > $S(dim)} then {
             return -code error -errorcode "0x30005" -errorinfo "Efficiency string number out of range"
        }
        set S(effic_$d) $e

        return [rpcvar::rpcvar string ""]
    }

    proc SpecCompressions {c1 c2} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecCompressions with parameters $c1 $c2"}
        set z [catch {Compressions $c1 $c2} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecCompressions detected error in call to Spectrum::Compressions"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SpecError {e} {
        global   MIDAS_SpectrumAccessServer_Verbose
        if {$MIDAS_SpectrumAccessServer_Verbose} {puts "In method SpecError with parameter $e"}
        set z [catch {Error $e} m]

        if {$z != 0} then {return -code error -errorcode "$m" -errorinfo "SpecError detected error in call to Spectrum::Error"}

        set status "$m"
        return [rpcvar::rpcvar string $status]
    }


    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export SpecDebug
     SOAP::export SpecLogging
     SOAP::export SpecNew
     SOAP::export SpecNew1D
     SOAP::export SpecNew2D
     SOAP::export SpecDelete
     SOAP::export SpecZero
     SOAP::export SpecDetails
     SOAP::export SpecNames
     SOAP::export SpecRead1DText
     SOAP::export SpecRead1D
     SOAP::export SpecRead1DR
     SOAP::export SpecRead2DText
     SOAP::export SpecRead2D
     SOAP::export SpecWrite1D
     SOAP::export SpecWrite2D
     SOAP::export SpecGetTitle
     SOAP::export SpecPutTitle
     SOAP::export SpecGetLabel
     SOAP::export SpecPutLabel
     SOAP::export SpecGetInfo
     SOAP::export SpecPutInfo
     SOAP::export SpecGetAnnot
     SOAP::export SpecPutAnnot
     SOAP::export SpecGetCalib
     SOAP::export SpecPutCalib
     SOAP::export SpecGetEffic
     SOAP::export SpecPutEffic
     SOAP::export SpecCompressions
     SOAP::export SpecError
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /SpectrumService -namespace urn:SpectrumService -uri urn:SpectrumService]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/SpectrumService {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

package provide SpectrumServer 1.0

#       set z [catch {interp eval "" ::namespace inscope $options(-namespace) $methodName $argValues} msg]
#       if {[llength $msg]==2} {
#          append xml "<status xsi:type=\"xsd:[lindex [set msg0 [lindex $msg 0]] 4]\">[lindex $msg0 5]</status>"
#          append xml "<return xsi:type=\"xsd:[lindex [set msg1 [lindex $msg 1]] 4]\">[lindex $msg1 5]</return>"
#       } else {
#          append xml "<return xsi:type=\"xsd:[lindex $msg 4]\">[lindex $msg 5]</return>"
#       }

