# Spectrum Addition application

# Mon Dec 13 18:57:58 GMT 1993, DB
#   started by copying cal.tcl and modifying

set started 0

set frame ADD
set frametitle "Adding Up Spectra"
set icontitle  SpecAdd

set factor ""
set name ""
set directory ""
set resultname ""
set resultdirectory ""
set summandlist {}
set summandstrings {}
set selection -1


proc repaint {} {
   global name factor directory resultname resultdirectory
    :modify FACTOR :value $factor
    :modify NAME   :value $name
    :modify DIRECTORY :value $directory
    :modify RESULT-NAME :value $resultname
    :modify RESULT-DIRECTORY :value $resultdirectory
}

proc format-summands l {
   set ll {}
   foreach f $l {lappend ll [format "%8.3f %12s   %-24s" \
      [first $f] [second $f] [third $f]]}
   return $ll
}

proc load-summand n {
   global name factor directory summandlist
   set summand [lindex $summandlist $n]
   set factor [first $summand]
   set name [second $summand]
   set directory [third $summand]
}

proc append-spectrum s {
   global debug summandlist
   if {$debug} then {puts stdout "append-spectrum $s"}
   set name [last [split $s "/"]]
   set directory [join [front [split $s "/"]] "/"]
   set factor 1.0
   lappend summandlist [list $factor $name $directory]
}

proc paint-summandlist {} {
   global summandlist summandstrings selection
   set summandstrings [format-summands $summandlist]
   :modify LIST :empty T :strings $summandstrings
   :modify LIST :select $selection
}

proc check-summand {} {
   global name factor directory
   if {[scan $factor "%f" factor] != 1} then {return 1}
   if {[scan $name "%s" name] != 1} then {return 1}
   if {[scan $directory "%s" directory] != 1} then {return 1}
   return 0
}

proc click-append {} {
   global factor name directory summandlist
   if {[check-summand] != 0} then \
      {set-footer "you haven't supplied enough data for a summand!!!"; return}
   lappend summandlist [list $factor $name $directory]
   paint-summandlist
   clear-footer
}

proc click-replace {} {
   global selection name factor directory summandlist
   if {$selection < 0} then \
      {set-footer "you haven't selected a summand to replace!"; return}
   set summandlist [lreplace $summandlist $selection $selection \
                     [list $factor $name $directory]]
   paint-summandlist
   clear-footer
}

proc click-delete {} {
   global selection summandlist
   if {$selection < 0} then \
      {set-footer "you haven't selected a summand to delete!"; return}
   set summandlist [lreplace $summandlist $selection $selection]
   set selection -1
   paint-summandlist
   clear-footer
}

proc click-delete-all {} {
   global selection summandlist
   set summandlist {}
   set selection -1
   paint-summandlist
   clear-footer
}

proc make-summand-exp {ss} {
   set ll {}
   foreach p $ss {
      append ll "[first $p]  [third $p]/[transparent [second $p]] "
   }
   return $ll
}

proc confirm-action {s} {
   set m "Spectrum $s already exists\nPlease confirm that you want to overwrite it"
   return [midas-confirmation $m]
}

proc click-addup {} {
   global resultdirectory resultname summandlist
   if {[llength $summandlist] == 0} then {
      set-footer "no summands selected!!!"
      return
   }
   if {$resultname == {}} then {
      set-footer "You haven't given a name for the result!!!"
      return
   }
   if {$resultdirectory == {}} then {
      set-footer "You haven't given a directory for the result!!!"
      return
   }
   set z [catch {eg "(spectrum :name '$resultdirectory/$resultname' ?spec)"}]
   if {$z == 0} then {
       if {[confirm-action $resultdirectory/$resultname] == 0} then {
          set-footer "action cancelled"
          return 
       }
   }
   set-footer "Adding spectra..."
   set z [catch {eval exec specaddition -result $resultdirectory/$resultname \
      -spectra [make-summand-exp $summandlist]} m]
   if {$z != 0} then {set-footer $m; return}
   set-footer "Result spectrum $resultdirectory/$resultname successfully created"
   send sb "redisplay-if-showing $resultdirectory"
}

proc click-list {v s} {
   global selection
   set selection $s
   :modify LIST :selection $s
   load-summand $s
   repaint
   clear-footer
}

proc click-factor {v} {
   global factor
   set z [scan $v "%f" zz]
   if {$z == 0} then {set-footer "$v is not an acceptable number"; return}
   set factor $v
   clear-footer
}

proc click-name {v} {
   global name
   set name $v
   clear-footer
}

proc click-directory {v} {
   global directory
   set directory $v
   clear-footer
}

proc click-result-name {v} {
   global resultname
   set resultname $v
   clear-footer
}

proc click-result-directory {v} {
   global resultdirectory
   set resultdirectory $v
   clear-footer
}

proc click-redisplay {} {
   paint-summandlist
   repaint
   clear-footer
}

proc start args {
   global started
   if {[llength $args] == 1} then {set args [first $args] }
   if {$started == 0} then {make-frame; set started 1} else {open-frame}
   foreach s $args {append-spectrum $s}
   click-redisplay
}


proc make-frame {} {
   global env axismenu ordermenu unitsmenu

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/spectrum"  \
      :icon-name [self icon-title]

    :panel P1 :layout L :pady 5 :padx 10
     :layout V; :row-gap 5
     :text FACTOR     :label Factor: :w 12 :valuex 10c
     :text NAME       :label Name:   :w 20 :valuex 10c
     :text DIRECTORY  :label Directory: :w 20 :valuex 10c
     :vskip 30
     :text RESULT-NAME :label Result:   :w 20 :valuex 10c
     :text RESULT-DIRECTORY :label Directory: :w 20 :valuex 10c

    :panel P2 :layout L :pady 5 :padx 5
     :layout V; :row-gap 10 
     :message M1 :label "     factor             name                         directory" :bold T
     :list LIST :rows 8 :cols 40 :exclusive T

    :panel P3 :layout B :after FOOTER :pady 5 :padx 10
     :layout H; :row-gap 10; :col-gap 5
     :button APPEND; :button REPLACE; :button DELETE
     :button DELETE-ALL :label "Delete All"
     :button ADDUP      :label "Add Up"
     :next-row T
     :button REDISPLAY
     :hskip 250
     :button HELP

  :show T
}
